#include "drawing.h"

usint min(usint a,usint b){
  if(a<b) return a;
  return b;
  }

usint max(usint a,usint b){
  if(a<b) return b;
  return a;
}

double absd(double d){
  if(d<0) return -d;
  return d;
}

int approx(double d){
  return (int)(d+0.5);
}

Color::Color(unsigned char r,unsigned char g,unsigned char b){
  red=r;green=g;blue=b;
}

Color::Color(unsigned char g){
  red=g;green=g;blue=g;
}

Screen::Screen(int w,int h){
  SDL_Init(SDL_INIT_VIDEO);
  sdl_screen=SDL_SetVideoMode(w, h, 32, SDL_HWSURFACE);
}

Screen::~Screen(){
  SDL_Quit();
}

void 
Screen::pause(){
  bool stop = false;
  SDL_Event event;
  while(!stop){
    SDL_WaitEvent(&event);
    switch(event.type){
    case SDL_QUIT:
      stop=true;;
    }
  }
}

void 
Screen::refresh(){
  SDL_Flip(sdl_screen);
}

void 
Screen::draw_pixel(usint x,usint y,Color c){
  if(x>=sdl_screen->w or y>=sdl_screen->h) return;
  *((Uint32*)(sdl_screen->pixels) +  x + (sdl_screen->h-y-1) * sdl_screen->w) =SDL_MapRGB(sdl_screen->format, c.red, c.green, c.blue);
}

void 
Screen::draw_line(PointSDL A,PointSDL B,Color c){
  //cout<<"A=("<<A.x<<","<<A.y<<") B=("<<B.x<<","<<B.y<<")"<<endl;
  if(A.x>sdl_screen->w or B.x>sdl_screen->w or A.y>sdl_screen->h or B.y>sdl_screen->h) return;
  int dx=B.x-A.x;
  if(dx<0){
    draw_line(B,A,c);
    return;
  }
  if(dx==0){
    usint ymin,ymax;
    ymin=min(A.y,B.y);
    ymax=max(A.y,B.y);
    for(usint y=ymin;y<=ymax;y++){
      draw_pixel(A.x,y,c);
    }
    return;
  }
  int dy=B.y-A.y;
  double m=double(dy)/double(dx);
  //cout<<"abs="<<abs(m)<<endl;
  if(absd(m)<=1){
    double y=A.y;
    // cout<<"ici"<<endl;
    for(usint x=A.x;x<=B.x;x++){
     
      draw_pixel(x,approx(y),c);
      y+=m;
    }
    return;
  }
  double x=A.x;
  m=1/m;
  if(dy>=0){
    for(usint y=A.y;y<=B.y;y++){
      draw_pixel(approx(x),y,c);
      x+=m;
    }
    return;
  }
  for(usint y=B.y;y<=A.y;y++){
    draw_pixel(approx(x),y,c);
    x+=m;
  }
}

Fenetre::Fenetre(double _xmin,double _xmax,double _ymin,double _ymax){
  xmin=_xmin;
  xmax=_xmax;
  ymin=_ymin;
  ymax=_ymax;
}

Transformation get_transformation(Fenetre F,Fenetre G){
  Transformation T;
  double rapport_x=(G.xmax-G.xmin)/(F.xmax-F.xmin);
  double rapport_y=(G.ymax-G.ymin)/(F.ymax-F.ymin);
  T.rapport=(rapport_x<rapport_y)?rapport_x:rapport_y;
  T.xtrans=G.xmin-F.xmin*T.rapport;
  T.ytrans=G.ymin-F.ymin*T.rapport;
  return T;
}

PointSDL transforme(Point P,Transformation T){
  PointSDL res;
  res.x=approx(P.x*T.rapport+T.xtrans);
  res.y=approx(P.y*T.rapport+T.ytrans);
  return res;
}

Fenetre Screen::get_fenetre(){
  return Fenetre(0,sdl_screen->w,0,sdl_screen->h);
}
  

void Screen::draw_polynome(Polynome P,Fenetre F,Color c){
  Fenetre F_SDL=get_fenetre();
  Point A;
  double xinc;
  A.x=F.xmin;
  xinc=(F.xmax-F.xmin)/(F_SDL.xmax-F_SDL.xmin);
  Transformation T=get_transformation(F,F_SDL);
  PointSDL Asdl;
  PointSDL AsdlP;
  A.x=F.xmin;
  A.y=P(A.x);
  Asdl=transforme(A,T);   
  for(A.x=F.xmin;A.x<=F.xmax;A.x+=xinc){
    AsdlP=Asdl;
    A.y=P(A.x);
    Asdl=transforme(A,T);
    draw_line(AsdlP,Asdl,c);
    //draw_pixel(Asdl.x,Asdl.y,Color(255));
  }

}





















