#include "polynome.h"

Polynome::Polynome(){
  degree=0;
  coeffs=new Rationnel[1];
  coeffs[0]=0;
}

Polynome::Polynome(usint n){
  degree=n;
  coeffs=new Rationnel[n+1];
  for(usint d=0;d<n;d++) coeffs[d]=0;
  coeffs[n]=1;
}

void Polynome::normalise(){
  usint d=degree;
  while(coeffs[d]==0 and d>=1) d--;
  if(d!=degree){
    Rationnel* newcoeffs=new Rationnel[d+1];
    for(usint k=0;k<=d;k++){
      newcoeffs[k]=coeffs[k];
    }
    delete[] coeffs;
    coeffs=newcoeffs;
    degree=d;
  }
}

Polynome::Polynome(const Polynome& P){
  degree=P.degree;
  coeffs=new Rationnel[degree+1];
  for(usint k=0;k<=degree;k++){
    coeffs[k]=P.coeffs[k];
  }
}

Polynome& Polynome::operator=(const Polynome& P){
  if(coeffs==P.coeffs) return *this;
  delete[] coeffs;
  degree=P.degree;
  coeffs=new Rationnel[degree+1];
  for(usint k=0;k<=degree;k++){
    coeffs[k]=P.coeffs[k];
  }
  return *this;
}

int Polynome::get_degree() const{
  if(degree>0) return degree;
  if(coeffs[0]==0) return -1;
  return 0;
}

Rationnel Polynome::operator()(const Rationnel& r) const{
  const Polynome& P=*this;
  Rationnel p=1;
  Rationnel res=0;
  for(usint d=0;d<=degree;d++){
    res=res+P[d]*p;
    p=p*r;
  }
  return res;
}

double Polynome::operator()(double r) const{
  const Polynome& P=*this;
  double p=1;
  double res=0;
  for(usint d=0;d<=degree;d++){
    res=res+P[d].to_double()*p;
    p=p*r;
  }
  return res;
}

Rationnel Polynome::operator[](usint i) const{
  if(i>degree) throw("Depacement d'indice");
  return coeffs[i];
}

Rationnel& Polynome::operator[](usint i){
  if(i>degree) throw("Depacement d'indice");
  return coeffs[i];
}


Polynome Polynome::operator+(const Rationnel& r) const{
  Polynome res(*this);
  res[0]=res[0]+r;
  return res;
}

Polynome Polynome::operator+(const Polynome& Q) const{
  const Polynome& P=*this;
  if(P.degree<Q.degree) return Q+P;
  usint d;
  Polynome res(P.degree);
  for(d=0;d<=Q.degree;d++) res[d]=P[d]+Q[d];
  for(;d<=P.degree;d++) res[d]=P[d];
  res.normalise();
  return res;
}

Polynome Polynome::operator-(const Rationnel& r) const{
  Polynome res(*this);
  res[0]=res[0]-r;
  return res;
}

Polynome Polynome::operator-(const Polynome& Q) const{
  const Polynome& P=*this;
  usint d;
  if(P.degree>Q.degree){
    Polynome res(P.degree);
    for(d=0;d<=Q.degree;d++) res[d]=P[d]-Q[d];
    for(;d<=P.degree;d++) res[d]=P[d];
    res.normalise();
    return res;
  }
  Polynome res(Q.degree);
  for(d=0;d<=P.degree;d++) res[d]=P[d]-Q[d];
  for(;d<=Q.degree;d++) res[d]=-Q[d];
  res.normalise();
  return res;
}

Polynome Polynome::operator-() const{
  const Polynome& P=*this;
  Polynome res(P.degree);
  usint d;
  for(d=0;d<=P.degree;d++) res[d]=-P[d];
  return res;
}

Polynome Polynome::operator*(const Rationnel& r) const{
  Polynome res(degree);
  const Polynome& P=*this;
  for(usint i=0;i<=degree;i++){
    res[i]=r*P[i];
  }
  res.normalise();
  return res;
}

Polynome Polynome::operator/(const Rationnel& r) const{
  Polynome res(degree);
  const Polynome& P=*this;
  for(usint i=0;i<=degree;i++){
    res[i]=P[i]/r;
  }
  return res;
}

Polynome Polynome::operator*(const Polynome& Q) const{
  const Polynome& P=*this;
  Polynome res(P.degree+Q.degree);
  for(usint k=0;k<=P.degree+Q.degree;k++){
    Rationnel c=0;
    for(usint i=0;i<=k;i++){
      if(i<=P.degree and k-i<=Q.degree) c=c+P[i]*Q[k-i];
    }
    res[k]=c;
  }
  return res;
}

Polynome operator+(const Rationnel& r,const Polynome& P){
  return P+r;
}

Polynome operator-(const Rationnel& r,const Polynome& P){
  Polynome Q(0);
  Q[0]=r;
  return Q-P;
}

Polynome operator*(const Rationnel& r,const Polynome& P){
  return P*r;
}

void Polynome::affiche_puissance(ostream& os,usint i) const{
  switch(i){
  case 0:
    break;
  case 1:
    os<<"X";
    break;
  default:
    os<<"X^"<<i;
    break;
  }
}

void Polynome::affiche_premier_monome(ostream& os,usint i) const{
  if(i==0){
    os<<"0";
    return;
  }
  Rationnel c=coeffs[i];
  if(c==1) return affiche_puissance(os,i);
  if(c==-1){
    os<<"-";
    affiche_puissance(os,i);
    return;
  }
  os<<c<<".";
  affiche_puissance(os,i);
}

void Polynome::affiche_monome(ostream& os,usint i) const{
  Rationnel c=coeffs[i];
  if(c==0) return;
  if(i==0){
    if(c>0){
      os<<"+";
    }
    os<<c;
    return;
  }
  if(c==1){
    os<<"+";
    affiche_puissance(os,i);
    return;
  }
  if(c==-1){
    os<<"-";
    affiche_puissance(os,i);
    return;
  }
  if(c>0) os<<"+";
  os<<c<<".";
  affiche_puissance(os,i);
  return;
}


ostream& operator<<(ostream& os,const Polynome& P){
  usint d=P.degree;
  P.affiche_premier_monome(os,d);
  for(;d>0;d--){
    P.affiche_monome(os,d-1);
  }
  return os;
}


