#include "rationnel.h"

long int pgcd(long int a,long int b){
  if(a<b) return pgcd(b,a);
  long int va,vb,r;
  va=a;vb=b;
  do{
    r=va%vb;
    va=vb;
    vb=r;
  }while(r!=0);
  return va;
}

Rationnel::Rationnel(){
  p=0;q=1;
}

Rationnel::Rationnel(long int _p){
  p=_p,q=1;
}

Rationnel::Rationnel(long int _p,long int _q){
  p=_p;q=_q;
  normalise();
}

Rationnel::Rationnel(const Rationnel& r){
  p=r.p;
  q=r.q;
}

Rationnel& Rationnel::operator=(const Rationnel& r){
  p=r.p;
  q=r.q;
  return *this;
}

Rationnel Rationnel::operator+(const Rationnel& r) const{
  Rationnel res;
  res.p=r.q*p+q*r.p;
  res.q=q*r.q;
  res.normalise();
  return res;
}

Rationnel Rationnel::operator-(const Rationnel& r) const{
  Rationnel res;
  res.p=r.q*p-q*r.p;
  res.q=q*r.q;
  res.normalise();
  return res;
}

Rationnel Rationnel::operator-() const{
  Rationnel res;
  res.p=-p;
  res.q=q;
  return res;
}

Rationnel Rationnel::operator*(const Rationnel& r) const{
  Rationnel res;
  res.p=p*r.p;
  res.q=q*r.q;
  res.normalise();
  return res;
}

Rationnel Rationnel::operator/(const Rationnel& r) const{
  Rationnel res; 
  if(r.p==0) throw("Division par zero");
  res.p=p*r.q;
  res.q=q*r.p;
  res.normalise();
  return res;
}


bool Rationnel::operator==(const Rationnel& r) const{
  return p==r.p and q==r.q;
}

bool Rationnel::operator!=(const Rationnel& r) const{
  return p!=r.p or q!=r.q;
}

bool Rationnel::operator<(const Rationnel& r) const{
  return p*r.q<q*r.p;
}

bool Rationnel::operator>(const Rationnel& r) const{
  return p*r.q>q*r.p;
}

bool Rationnel::operator<=(const Rationnel& r) const{
  return p*r.q<=q*r.p;
}

bool Rationnel::operator>=(const Rationnel& r) const{
  return p*r.q>=q*r.p;
}

void Rationnel::normalise(){
  if(q<0){
    p=-p;
    q=-q;
  }
  if(p==0){
    q=1;
    return;
  }
  long int d;
  d=pgcd(p,q);
  if(d<0) d=-d;
  p=p/d;
  q=q/d;
}

double Rationnel::to_double() const{
  return ((double)p)/((double)q);
}

Rationnel operator+(const long int& a,const Rationnel& r){
  return Rationnel(a)+r;
}

Rationnel operator-(const long int& a,const Rationnel& r){
  return Rationnel(a)-r;
}

Rationnel operator*(const long int& a,const Rationnel& r){
  return Rationnel(a)*r;
}

Rationnel operator/(const long int& a,const Rationnel& r){
  return Rationnel(a)/r;
}

bool operator==(const long int& a,const Rationnel& r){
  return Rationnel(a)==r;
}

bool operator!=(const long int& a,const Rationnel& r){
  return Rationnel(a)!=r;
}

bool operator<(const long int& a,const Rationnel& r){
  return Rationnel(a)<r;
}

bool operator>(const long int& a,const Rationnel& r){
  return Rationnel(a)>r;
}

bool operator<=(const long int& a,const Rationnel& r){
  return Rationnel(a)<=r;
}

bool operator>=(const long int& a,const Rationnel& r){
  return Rationnel(a)>=r;
}

ostream& operator<<(ostream& os,const Rationnel& r){
  if(r.p==0){
    os<<'0';
    return os;
  }
  if(r.q==1){
    os<<r.p;
    return os;
  }
  os<<r.p<<"/"<<r.q;
  return os;
}



